# This script runs all the other scripts needed for the analysis in the appropriate order.
# You must set the working directory to wherever the scripts are located:

setwd("/home/stats/wolfch/frag_geo/project_code_v002")

dir.create("output") # create directory to store output

require(pbapply)
require(plyr)
require(stringr)
require(countrycode)
require(foreach)
require(doSNOW)
require(glmmTMB)
require(boot)
require(broom)
require(broom.mixed)
require(reshape2)
require(Hmisc)
require(ggplot2)
require(RColorBrewer)
require(grid)
require(taxize)
require(RCurl)
require(stringr)
require(scales)
require(gridExtra)
require(lme4)
require(cowplot)
require(stringi)
require(readxl)
require(ggrepel)
require(png)
require(rgdal)
require(NLMR)
require(landscapetools)
require(raster)
require(brms)
require(unixtools)
require(rstan)
require(openxlsx)
require(accelerometry)

######################### load utility functions for model fitting

source("myPredict.R") # fix issue w/ glmmTMB looking for weights in original data when predicting

# http://r.789695.n4.nabble.com/warning-inside-loop-td1011667.html
# https://stats.stackexchange.com/questions/343990/how-can-i-extract-warning-and-error-messages-from-nlme-model-object
# utility function to record for warnings when fitting models:
withWarnings = function (expr)
{
    warnings <- character()
    retval <- withCallingHandlers(expr, warning = function(ex) {
        warnings <<- c(warnings, conditionMessage(ex))
        invokeRestart("muffleWarning")
    })
    list(Value = retval, Warnings = warnings)
}

######################### a simple example

data = readRDS("input/model_data.RDS")
data = data[data$type == "Forest",] # forest species only
data$y = as.numeric(1:nrow(data) %in% grep("Core", data$Category)) # core/non-core as response
mod = glm(y ~ any, family="binomial", data=data) # model effect of any disturbance for all forest species
summary(mod)

# note: this example model was not used in our actual analysis, but is just provided
# as a simple example of modeling the effect of disturbance on the likelihood of forest
# species being classified as "core".

######################### run analysis scripts

# fit the main disturbance models
source("001 fit main disturbance models.R")

# fit latitude models
source("002 fit latitude models.R")

# plot primary model results - Figure 2
source("003 plot main disturbance results.R")

# plot absolute latitude results model results - Figures 3, S8
source("004 plot main lat results.R")

# map landscapes by class and for all species - Figure S1
source("005 map landscapes.R")

# look at core habitat amount etc. for simulated landscapes - Figures S2, S3
source("006 landscape simulation.R")

# forest / matrix / generalist versus disturbance models
source("007 multinomial habitat - disturbance.R")

# forest / matrix / generalist versus disturbance results - Figure S5, Table S4 (half)
source("008 multinomial habitat - disturbance - results.R")

# forest / matrix / generalist versus latitude models
source("009 multinomial habitat - latitude.R")

# forest / matrix / generalist versus latitude results - Figure S9, Table S4 (half)
source("010 multinomial habitat - latitude - results.R")

# sensitivity analysis models with order and family as random effects, etc.
source("011 fit models - order family.R")

# sensitivity analysis results - order+family and dropping duplicates (disturbance) - Figure S7
source("012 sensitivity - disturbance.R")

# sensitivity analysis results - latitude with order + family etc. - Figure S10
source("013 sensitivity - latitude.R")

# sample sizes, Core/Edge/noPref summary, paper text results - Tables S1, S2
source("014 sample size tables.R")

# main model results tables (disturbance, latitude) - Tables ????????????????
source("015 disturbance results table.R")
source("016 latitude results table.R")

# model and make table for non-migratory birds sensitivity analysis - Table S5
source("017 migratory status models.R")
source("018 migratory status table.R")

# models and table for range size and distance to edge of range etc. sensitivity analysis - Table S6
source("019 range size dist models.R")
source("020 range size dist table.R")

# models and table for habitat amount sensitivity analysis - Tables S7, S8
source("021 habitat amount models crosstab.R")
source("022 habitat amount table.R")

