data = readRDS("input/model_data.RDS")

data$group = data$class_simple # UPDATE
data$y = as.numeric(1:nrow(data) %in% grep("Core", data$Category))
data$pretty_label = paste0(data$Location_Country, ": ", data$Taxon)

data_all = data
data_all$class_simple = "All species"
data_class = rbind(data, data_all)

######################### models

model_mat = expand.grid(class_simple=unique(data_class$class_simple),
	covars=list("any","natural","hist_loss_binary"),
	type="Forest",
        species_policy=c("Random effect","Drop duplicates","none"),
	stringsAsFactors=FALSE)
model_mat$rel_tol = 1e-7
model_mat$rel_tol[model_mat$species_policy == "none"] = 1e-10

cl = makeCluster(8, type = "SOCK", outfile="")
registerDoSNOW(cl)
out = foreach(i=1:nrow(model_mat), .packages=c("boot","glmmTMB","broom","broom.mixed")) %dopar% {
	print(paste0("STARTING: ", i))
	attach(model_mat[i,])
	assignInNamespace("predict.glmmTMB",myPredict, ns="glmmTMB") # substitute fixed function (foreach loads glmmTMB each time)

	df = data_class[data_class$class_simple %in% class_simple & data_class$type %in% type,]
        if(species_policy == "Drop duplicates")
                df = df[! duplicated(df$Species.name),]

	if(nrow(df) == 0 | length(unique(df$name)) == 1) return(NULL)
        if("any" %in% covars[[1]] & min(table(df$any, df$y)) == 0) return(NULL) # complete separation
        if("natural" %in% covars[[1]] & min(table(df$natural, df$y)) == 0) return(NULL) # complete separation
        if("hist_loss_binary" %in% covars[[1]] & min(table(df$hist_loss_binary, df$y)) == 0) return(NULL)

        form = paste0("cbind(y,1-y) ~ ", paste(covars[[1]],collapse="+"), " + (1 | name)")
        if(class_simple == "All species")
        	form = paste0(form, " + (1 | group)")
        if(species_policy == "Random effect" & class_simple != "Herptiles")
        	form = paste0(form, " + (1 | Species.name)")
        form = as.formula(form)

        # can look into r2 e.g. https://stat.ethz.ch/pipermail/r-sig-mixed-models/2018q1/026405.html
	mod = withWarnings(glmmTMB(form, weights=df$dataset.rating,
		data=df, family=binomial(link = "logit"),
                           control = glmmTMBControl(optCtrl=list(rel.tol=rel_tol))))
        warn_main = mod$Warnings
        mod = mod$Value
        if(length(grep("onvergence", warn_main)) > 0) return(NULL)

	### predict w/ and w/o disturbance
        pred_dist = NULL
        if(paste(covars[[1]], collapse=";") == "any"){
                newdata = data.frame(y=0, any=c(FALSE,TRUE), name="new", `(weights)`=1, check.names=FALSE)
                # https://stat.ethz.ch/pipermail/r-sig-mixed-models/2017q4/026150.html
                beta = summary(mod)$coef$cond[,1] # for backtransforming        
                X = model.matrix(y ~ any, data=newdata) # model matrix (no random effects)
                pred_logit = X %*% beta
                se_logit = sqrt(diag(X %*% vcov(mod)[["cond"]] %*% t(X))) # Just use V(X*beta_hat) = X*V(beta_hat)*X'
                pred_dist = data.frame(model_mat[i,],
                             newdata,
                             n=as.numeric(table(df$any)),
                             estimate=inv.logit(pred_logit),
                             lower=inv.logit(pred_logit - qnorm(0.975) * se_logit),
                             upper=inv.logit(pred_logit + qnorm(0.975) * se_logit))
        }

        ### output

	out = list(mod_summary=data.frame(id=i, model_mat[i,], n=nrow(df), tidy(mod, effect="fixed"), n_warn=length(warn_main), AIC=AIC(mod)),
                pred_dist=pred_dist,
                settings=model_mat[i,])
        out$mod_summary$covars = paste(covars[[1]], collapse=";")
	print(paste0("FINISHED: ", i))
	return(out)
}
stopCluster(cl)

out_col = list(mod_tab=do.call("rbind",lapply(out,function(x) x$mod_summary)),
        pred_dist=do.call("rbind",lapply(out,function(x) x$pred_dist)))

out_col$mod_tab = out_col$mod_tab[out_col$mod_tab$term != "(Intercept)" & out_col$mod_tab$species_policy == "Random effect",]

out_col$mod_tab$p_adj = p.adjust(out_col$mod_tab$p.value, method="fdr")
mod_tab = out_col$mod_tab

saveRDS(out_col, "output/models_pred_disturbance_species_policy.RDS")

