set.seed(2)
example_rasts = foreach(dimension=c(0.1,0.5,1.5), .combine="rbind") %do% {
  cont_landscape = nlm_fbm(100, 100, fract_dim = dimension)
  out = foreach(habitat=c(0.95,0.8,0.2), .combine="rbind") %do% {
    disc_landscape = landscapetools::util_classify(cont_landscape, weighting = c(1 - habitat, habitat)) - 1
    disc_landscape[disc_landscape[] %in% 0] = NA
    edge_landscape = boundaries(disc_landscape, type='inner', classes=FALSE, directions=8)
    edge_landscape[is.na(edge_landscape[])] = -1
    out = data.frame(rasterToPoints(edge_landscape),
                     Habitat=paste0(100*habitat,"%"),
                     Dimension=dimension)
    return(out)
  }
  return(out)
}

example_rasts$Habitat = factor(example_rasts$Habitat, levels=unique(example_rasts$Habitat))

p = ggplot(example_rasts, aes(x=x,y=y,fill=factor(layer))) +
  facet_grid(Habitat ~ Dimension, switch="y", labeller = label_both) +
  geom_raster() +
  scale_fill_manual(values=c("purple","darkgreen","#ffffcc"),
                    breaks=c(0,1,-1),
                    labels=c("Habitat core","Habitat edge","Matrix"),
                    guide=guide_legend(title=NULL)) +
  theme_bw() +
  scale_x_continuous(expand=c(0,0)) +
  scale_y_continuous(expand=c(0,0)) +
  theme(axis.text=element_blank(),
        axis.ticks=element_blank(),
        axis.title=element_blank(),
        legend.background = element_rect(color="black"),
        legend.position="bottom")

pdf("output/examples.pdf", width=7, height=7)
print(p)
dev.off()
 
png("output/examples.png", width=7, height=7, units="in", res=300)
print(p)
dev.off()

############################# simulation

set.seed(1)
sim_data = foreach(dimension=rep(c(0.05,0.5,1,1.5),each=10), .combine="rbind") %do% {
  print(dimension)
  cont_landscape = nlm_fbm(102, 102, fract_dim = dimension)
  out = foreach(habitat=seq(0,1,by=0.05), .combine="rbind") %do% {
    disc_landscape = landscapetools::util_classify(cont_landscape, weighting = c(1 - habitat, habitat)) - 1
    disc_landscape[disc_landscape[] %in% 0] = NA
    edge_landscape = boundaries(disc_landscape, type='inner', classes=FALSE, directions=8)
    prop_landscape = ifelse(habitat != 1, sum(edge_landscape[2:101,2:101] %in% 0) / 1e4, 1)
    out = data.frame(prop_landscape=prop_landscape,
                     Habitat=habitat,
                     Dimension=dimension)
    return(out)
  }
  return(out)
}
sim_data$Dimension = factor(sim_data$Dimension, levels=rev(unique(sim_data$Dimension)))

sim_avgs = aggregate(prop_landscape ~ Habitat + Dimension, data=sim_data, FUN="mean")

p = ggplot(sim_data, aes(x=Habitat,y=prop_landscape,color=Dimension,fill=Dimension)) +
  geom_point() +
  scale_x_reverse(limits=c(1,0), expand=c(0,0)) +
  geom_abline(intercept=0,slope=-1, color="black", size=1.5, linetype="dashed") +
  theme_bw() +
  geom_line(data=sim_avgs, size=1) +
  xlab("Landscape-scale habitat amount") +
  ylab("Proportion core habitat in the landscape (total abundance)") +
  theme(axis.text=element_text(color="black"),
        axis.ticks=element_line(color="black"),
        panel.border=element_rect(color="black"),
        legend.position=c(0.95,0.95),
        legend.justification=c(1,1),
        legend.background = element_rect(color="black"),
        plot.margin=margin(c(5,15,5,5))) +
  scale_y_continuous(limits=c(-1e-2,1),expand=c(0,0)) +
  scale_color_manual(values=brewer.pal(5,"Set1")[-1],guide=guide_legend(title="Fractal\ndimension")) +
  scale_fill_manual(values=brewer.pal(4,"Set1"),guide=guide_legend(title="Fractal\ndimension"))

pdf("output/sim_results2.pdf", width=7, height=7)
print(p)
dev.off()
 
png("output/sim_results2.png", width=7, height=7, units="in", res=300)
print(p)
dev.off()

