data = readRDS("input/model_data.RDS")
data$class_raw = data$class_simple

data_all = data
data_all$class_simple = "All species"
data_class = rbind(data, data_all)

data_class$type = factor(data_class$type,
        levels=c("Forest","Matrix","Generalist")) # when we subset, can lose some levels o.w.
data_class = data_class[! is.na(data_class$type),]

######################### models

model_mat = expand.grid(class_simple=unique(data_class$class_simple),
	covars=list("any"),
	stringsAsFactors=FALSE)

cl = makeCluster(5, type = "SOCK", outfile="")
registerDoSNOW(cl)
clusterCall(cl, function(){
        require(unixtools)                     
        dir.create(t <- paste0("temp/", Sys.getpid()), FALSE, TRUE, "0700")
        set.tempdir(t)
})
out = foreach(i=1:nrow(model_mat), .packages=c("rstan","brms","reshape2")) %dopar% {
	print(paste0("STARTING: ", i))
	attach(model_mat[i,])

	df = data_class[data_class$class_simple %in% class_simple,]

	newdata = data.frame(any = unique(df$any))
        newdata$name = "new_gp"

        if(length(unique(df$type)) == 2){
                if(class_simple != "All species"){
                        mod = brm(type ~ any + (1 | name) + (1 | Species.name), # Forest < Matrix, so we predict P(Matrix)..
                        algorithm = "meanfield",
                        save_ranef = FALSE, save_dso = FALSE, elbo_samples = 2500, tol_rel_obj=0.001, adapt_iter=500, grad_samples=5, seed=1,
                        prior = prior(normal(0,5), class = Intercept) +
                                prior(normal(0,5), class = b) +
                                prior(normal(0,1), class = sd),
                        data = df,
                        family = bernoulli())
                } else{
                        mod = brm(type ~ any + (1 | name) + (1 | class_raw) + (1 | Species.name), # Forest < Matrix, so we predict P(Matrix)..
                        algorithm = "meanfield",
                        save_ranef = FALSE, save_dso = FALSE, elbo_samples = 2500, tol_rel_obj=0.001, adapt_iter=500, grad_samples=5, seed=1,
                        prior = prior(normal(0,5), class = Intercept) +
                                prior(normal(0,5), class = b) +
                                prior(normal(0,1), class = sd),
                        data = df,
                        family = bernoulli())
                }
                pred = data.frame(fitted(mod, newdata = newdata, allow_new_levels=TRUE))
                pred = cbind(1-pred, pred, x1=NA, x2=NA, x3=NA, x4=NA) # no generalist plants!; Order should be F, M, G
        } else{
                if(class_simple != "All species"){
                        mod = brm(type ~ any + (1 | name) + (1 | Species.name),
                        algorithm = "meanfield", # "fullrank" did not converge by 1e4 iter (i=6)
                        save_ranef = FALSE, save_dso = FALSE, elbo_samples = 2500, tol_rel_obj=0.001, adapt_iter=500, grad_samples=5, seed=1,
                        prior = prior(normal(0,5), class = Intercept) + 
                                prior(normal(0,5), class = b) +
                                prior(normal(0,1), class = sd, dpar="muGeneralist") +
                                prior(normal(0,1), class = sd, dpar="muMatrix"),                        
                        data = df,
                        family = categorical())
                } else{
                        mod = brm(type ~ any + (1 | name) + (1 | class_raw) + (1 | Species.name),
                        algorithm = "meanfield",
                        save_ranef = FALSE, save_dso = FALSE, elbo_samples = 2500, tol_rel_obj=0.001, adapt_iter=500, grad_samples=5, seed=1,
                        prior = prior(normal(0,5), class = Intercept) + 
                                prior(normal(0,5), class = b) +
                                prior(normal(0,1), class = sd, dpar="muGeneralist") +
                                prior(normal(0,1), class = sd, dpar="muMatrix"),                        
                        data = df,
                        family = categorical())
                }
                # use "fitted" instead of "predict", since we don't want to perform a posterior predictive check..
                pred = data.frame(fitted(mod, newdata = newdata, re_formula=NA, allow_new_levels=TRUE))
                # note -- we want CI for the true population mean proportion across studies, so no need to include r.e.
                # Would include r.e. if we wanted bands for which there was a 95% probability that A NEW STUDY/DATASET
                # would have mean proportion in there.  Our goal for inference is to assess the overall average relationship,
                # rather than capture the means for a new study.
        }

        colnames(pred)[c(1,5,9)] = c("Forest", "Matrix", "Generalist")
        pred_melt = melt(pred[,c(1,5,9)], id.vars=NULL)
        pred_melt$lower = melt(pred[,c(1,5,9) + 2], id.vars=NULL)$value
        pred_melt$upper = melt(pred[,c(1,5,9) + 3], id.vars=NULL)$value
        colnames(pred_melt)[1:2] = c("type", "EI.sensitivity")
        pred = pred_melt
	pred$class_simple = class_simple
        pred$any = rep(newdata$any, times=3) # messy but whatever
        pred = merge(pred, melt(table(type=df$type, any=df$any), value.name="n"))

       	### output
	out = list(mod=mod,
                pred=pred)
	print(paste0("FINISHED: ", i))
	return(out)
}
stopCluster(cl)

saveRDS(out, "output/models_pred_multinomial_habitat_disturbance.RDS")

############### The odds of a of species being forest-associated were xx.x% [95% CI: xx.x%, xx.x%] higher in low disturbance
# versus high-disturbance regions 

summary(out[[5]]$mod)

samps = posterior_samples(out[[5]]$mod,
                          pars = paste0("b_",c("muMatrix_Intercept","muGeneralist_Intercept","muMatrix_abs_lat","muGeneralist_abs_lat")))

newdata = data.frame(any=FALSE, name = "new_gp") # can add class_raw = "new_class" - doesn't matter (r.e.'s have mean 0)
probs_low = fitted(out[[5]]$mod, newdata = newdata, re_formula=NA, allow_new_levels=TRUE, summary=FALSE)
p_forest_low = probs_low[1:1000,,][,1]

newdata = data.frame(any=TRUE, name = "new_gp")
probs_high = fitted(out[[5]]$mod, newdata = newdata, re_formula=NA, allow_new_levels=TRUE, summary=FALSE)
p_forest_high = probs_high[1:1000,,][,1]

odds_ratio = (p_forest_low/(1-p_forest_low))/(p_forest_high/(1-p_forest_high))

quantile(odds_ratio, c(0.025,0.5,0.975))
100 * (quantile(odds_ratio, c(0.025,0.5,0.975)) - 1)

