######################### basic setup

data = readRDS("input/model_data.RDS")

data$group = data$class_simple # UPDATE
data$y = as.numeric(1:nrow(data) %in% grep("Core", data$Category))

data_all = data
data_all$class_simple = "All species"
data_class = rbind(data, data_all)

######################### models

model_mat = expand.grid(class_simple=unique(data_class$class_simple),
	covars=list("abs_lat","any"),
	type="Forest",
	stringsAsFactors=FALSE)

cl = makeCluster(8, type = "SOCK", outfile="")
registerDoSNOW(cl)
out = foreach(i=1:nrow(model_mat), .packages=c("boot","glmmTMB","broom","broom.mixed")) %dopar% {
	print(paste0("STARTING: ", i))
	attach(model_mat[i,])
	assignInNamespace("predict.glmmTMB",myPredict, ns="glmmTMB") # substitute fixed function (foreach loads glmmTMB each time)

	df = data_class[data_class$class_simple %in% class_simple & data_class$type %in% type,]
        no_family = sum(is.na(df$family))
        df = df[! is.na(df$family),] # have order whenever there's family...

	if(nrow(df) == 0 | length(unique(df$name)) == 1 | length(unique(df$order)) == 1) return(NULL)
        if("any" %in% covars[[1]] & min(table(df$any, df$y)) == 0) return(NULL) # complete separation
        if("natural" %in% covars[[1]] & min(table(df$natural, df$y)) == 0) return(NULL)
        # %-- complete separation (?).. seems bad even w/ random effect(s)
        if("hist_loss_binary" %in% covars[[1]] & min(table(df$hist_loss_binary, df$y)) == 0) return(NULL)

	form = as.formula(paste0("cbind(y,1-y) ~ ", paste(covars[[1]],collapse="+"), " + (1 | order) + (1 | family) + (1 | name) + (1 | Species.name)"))
        if(class_simple == "All species")
        	form = as.formula(paste0("cbind(y,1-y) ~ ",
                                         paste(covars[[1]],collapse="+"), " + (1 | order) + (1 | family) + (1 | name) + (1 | group) + (1 | Species.name)")) # UPDATE
	mod = withWarnings(glmmTMB(form, weights=df$dataset.rating, control = glmmTMBControl(optCtrl=list(rel.tol=1e-7)),
		data=df, family=binomial(link = "logit"))) # look into r2 e.g. https://stat.ethz.ch/pipermail/r-sig-mixed-models/2018q1/026405.html
        warn_main = mod$Warnings
        mod = mod$Value
        if(length(grep("onvergence", warn_main)) > 0) return(NULL)

	### predict (wrt latitude)
        pred = NULL
        if(paste(covars[[1]], collapse=";") == "abs_lat"){
	n_pred = 100
	newdata = data.frame(class_simple = class_simple,
		type = type,
                group = "NEW_GROUP",
		name = "NEW_STUDY",
                family = "NEW_FAMILY",
                order = "NEW_ORDER",
                Species.name = "NEW_SPECIES",
		abs_lat = seq(min(df$abs_lat), max(df$abs_lat), length=n_pred),
		forest_cover = mean(df$forest_cover,na.rm=TRUE),
		`(weights)` = 1, # weights don't affect prediction/se (tested),
		covars = paste(covars[[1]], collapse=";"), # so we know what the prediction is for..
		check.names = FALSE
	)

	pred_info = predict(mod, newdata=newdata, allow.new.levels=TRUE, se.fit=TRUE)

        ### switch from prediction se to confidence se (narrower) -- see details in the disturbance model script
        newdata$y = 0
        X = model.matrix(y ~ abs_lat, data=newdata) # model matrix (no random effects)
        se_logit = sqrt(diag(X %*% vcov(mod)[["cond"]] %*% t(X)))
        pred_info[[2]] = se_logit

	pred = data.frame(newdata,
		EI.sensitivity = inv.logit(pred_info[[1]]),
		lower = inv.logit(pred_info[[1]] - qnorm(0.975)*pred_info[[2]]),
		upper = inv.logit(pred_info[[1]] + qnorm(0.975)*pred_info[[2]]),
		p.value = tidy(mod)$p.value[tidy(mod)$term == rev(covars[[1]])[1]]
	)
        }

	### predict w/ and w/o disturbance
        pred_dist = NULL
        if(paste(covars[[1]], collapse=";") == "any"){
        newdata = data.frame(y=0, any=c(FALSE,TRUE), name="new", `(weights)`=1, check.names=FALSE)
        # v-- https://stat.ethz.ch/pipermail/r-sig-mixed-models/2017q4/026150.html
        beta = summary(mod)$coef$cond[,1] # for backtransforming        
        X = model.matrix(y ~ any, data=newdata) # model matrix (no random effects)
        pred_logit = X %*% beta
        se_logit = sqrt(diag(X %*% vcov(mod)[["cond"]] %*% t(X)))
        pred_dist = data.frame(model_mat[i,],
                             newdata,
                             n=as.numeric(table(df$any)),
                             estimate=inv.logit(pred_logit),
                             lower=inv.logit(pred_logit - qnorm(0.975) * se_logit),
                             upper=inv.logit(pred_logit + qnorm(0.975) * se_logit))
        }

        ### output

	out = list(mod_summary=data.frame(id=i, model_mat[i,], n=nrow(df), tidy(mod, effect="fixed"),
                                          n_warn=length(warn_main), no_family=no_family,AIC=AIC(mod)),
		pred=pred,
                pred_dist=pred_dist,
                settings=model_mat[i,])
        out$mod_summary$covars = paste(covars[[1]], collapse=";")
	print(paste0("FINISHED: ", i))
	return(out)
}
stopCluster(cl)

out_col = list(mod_tab=do.call("rbind",lapply(out,function(x) x$mod_summary)),
	pred=do.call("rbind",lapply(out,function(x) x$pred)),
        pred_dist=do.call("rbind",lapply(out,function(x) x$pred_dist)))

out_col$mod_tab$p_adj = p.adjust(out_col$mod_tab$p.value, method="fdr")
mod_tab = out_col$mod_tab
out_col$pred$p_adj = mod_tab$p_adj[match(out_col$pred$p.value,mod_tab$p.value)]

out_col$pred$FDR_pval_cut = cut(out_col$pred$p_adj, breaks=c(0,.05,.1,1))
levels(out_col$pred$FDR_pval_cut) = paste0(levels(out_col$pred$FDR_pval_cut), " ")

saveRDS(out_col, "output/models_pred_disturbance_order_family.RDS")

